#!/bin/bash

# rough breakdown; all tests assert Name

# test1: asserts %i exec, exec, path,
# test2: asserts terminal

readonly OPENER=xdg-open
if ! type -P "$OPENER" &> /dev/null; then
	echo "couldn't find opener '$OPENER' correct binary name?  examples of valid openers being gnome-open, kfmclient..."
	exit -1
fi

install_mime() {
	xdg-mime "$@"
	[ -n $KDE ] && kbuildsycoca &> /dev/null
}

query() {
	echo "$@"
	ret=''
	while [ -z "$ret" ]; do
		echo -n "y/n? :"
		read ret
		if ! [ "$ret" == "y" -o "$ret" == "n" ]; then
			echo "invalid response; must be 'y' or 'n'"
			ret=''
		fi
	done
	[ "$ret" == "y" ] && return 0
	return 1
}	

log_failure() {
	echo ">> $TESTCASE $@"
}

log_pass() {
	:
}

test01_setup() {
	local phase="${1-install}"
	local size
	for size in icons/*; do
		sudo xdg-icon-resource ${phase} "${size}/desktoptestredx.png" --size "$(basename ${size})" --novendor
	done
	if [ -z "${phase}" ] || [ "$phase" == "uninstall" ]; then
		sudo xdg-desktop-menu ${phase} --mode system "$LOC"
		unset LOC
	else
		export LOC=$(make_mime_handler "$LOGGER_SH ${WORKDIR}/ret %i" 'Icon=desktoptestredx')
		sudo xdg-desktop-menu ${phase} --mode system ${LOC}
		echo $LOC
	fi
}

test01_teardown() {
	test01_setup uninstall
}

test01() {
	# basic exec verification, name verification, icon verification, path verification, terminal verification.
	local loc ret=0

	if ! query $'test01: please look in your menu for an entry named "Desktop Entry Spec Test1" and click on it if you can find it\nwas it found?'; then
		log_failure "desktop entry not found, can't test further"
		return 1
	fi
	icon_found="as"
	if ! query $'test01: did it have a red icon?'; then
		log_failure "icon wasn't used/found";
		unset icon_found
	fi
	if [ ! -e "${WORKDIR}/ret" ]; then
		log_failure "${WORKDIR}/ret doesn't exist, indicates exec failed";
		return 1
	elif [ -z "$(</${WORKDIR}/ret)" ]; then
		[ -n "$icon_found" ] && \
		log_failure "${WORKDIR}/ret: %i was empty, yet icon was found already"
		ret=1
	elif [ -z "$icon_found" ]; then
		log_failure "%i was expanded, yet user stated no icon showed"
		ret=1
	fi
	return $(( $ret ))
}


make_test_setup_pair() {
	eval "test${1}_setup() { ${2:-_exec_test_setup}; }"
	eval "test${1}_teardown() { ${2:-_exec_test_setup} uninstall; }"
}

_exec_test_setup() {
	install_mime ${1:-install} "${DATADIR}/desktop-spec-test.xml"
}

make_mime_handler() {
	cat "${DATADIR}/generic-exec-test.desktop" | sed -e "s|%EXEC%|${1}|" | sed -e "s|%EXTRAS%|${2}|" \
		| grep -v '^$' > "${WORKDIR}/${NAME-desktop-spec-test.desktop}"
	echo "${WORKDIR}/${NAME-desktop-spec-test.desktop}"
#	desktop-file-validate "${WORKDIR}/${NAME-desktop-spec-test.desktop}" 2>&1 | grep -v 'required key "Encoding"' >&2
}

newline_expand() {
	for x in $*; do
		echo $x;
	done
	unset x
}

make_test_setup_pair 02
test02() {
	# basic exec verification, name verification, icon verification, path verification, terminal verification.
	local ret=0
	local loc=$(make_mime_handler "$LOGGER_SH ${WORKDIR}/ret %F")
	local TEST2_BASE="$(dirname ${TESTDATA})/test2-data"
	xdg-desktop-menu install "$loc"
	install_mime default "$(basename "$loc")" "$TESTMIME"
	(
	xdg-open "${TEST2_BASE}"
	if ! query "test02: a directory should've just opened, please do not close it- did it open?"; then
		log_failure "xdg-open failed, can't test further"
		return 1;
	elif ! query "test02: please select all files, and open the files (for example via right clicking and selecting "open with")"; then
		log_failure "user intervention blocked testing plurals execution test"
		return 1
	fi
	# sleep for a few to ensure that the async handling of the event has completed.
	sleep ${SLEEPDELAY}
	if [ ! -e "${WORKDIR}/ret" ]; then
		log_failure "no results found in ${WORKDIR}/ret; exec didn't run it seems"
		return 1
	fi
	local fullpath="$(for x in 1 2 3; do echo "${TEST2_BASE}/file-$x.spec_desktop_test"; done)"
	local url="$(for x in 1 2 3; do echo "file://${TEST2_BASE}/file-$x.spec_desktop_test"; done)"
	local data=$(echo $(newline_expand $(cat ${WORKDIR}/ret) | sort -g))
	if [ "$data" != "$(echo $url)" ] && \
		[ "$data" != "$(echo $fullpath)" ]; then
		log_failure "didn't get valid results: $(echo $data); expected $fullpath or $url"
	fi
	)
	ret=$?
	xdg-desktop-menu uninstall "$loc"
	unset loc TEST2_BASE
	return $(( $ret ))
}


# automated tests...

generic_exec_test() {
	xdg-desktop-menu install "$1"
	# env protection, among other things
	(
		install_mime default "$(basename ${1})" $TESTMIME
		shift;
		${OPENER} "$@"
		# sleep for slow systems for testing; this is still semi-racey, but best we can do.
		sleep ${SLEEPDELAY}
		# set the exit return code via this logic test; the echo $(cat x) is a way to ignore whitespace differences also.
	)
	ret="$?"
	xdg-desktop-menu uninstall "$1"
	return $(( $ret ))
}	

make_test_setup_pair 10 _exec_test_setup
test10() {
	local loc ret=0
	loc=$(make_mime_handler "python -c'import os,sys;open(\"${WORKDIR}/ret\", \"w\").write(str(os.isatty(sys.stdout.fileno())))'" 'Terminal=true')
	generic_exec_test "$loc" "${TESTDATA}"
	if [ "$(echo $(cat "${WORKDIR}/ret" 2>&1))" != "True" ]; then
		log_failure "Terminal key isn't supported, got '$(<${WORKDIR}/ret)' instead of True"
		return 1
	fi
	:
}

make_test_setup_pair 11 _exec_test_setup
test11() {
	local loc=$(make_mime_handler "rm %f")
	touch "${WORKDIR}/f.${TEST_EXT}"
	generic_exec_test "$loc" "${WORKDIR}/f.${TEST_EXT}"
	if [ -e "${WORKDIR}/f.${TEST_EXT}" ]; then
		log_failure "exec %f parsing failed; test file wasn't wiped."
		return 1
	fi
	:
}

make_test_setup_pair 12 _exec_test_setup
disabled_test12() {
	local loc=$(make_mime_handler "$LOGGER_SH ${WORKDIR}/ret %d")
	generic_exec_test "$loc" "${TESTDATA}"
	if [ "$(echo $(cat "${WORKDIR}/ret"))" != "$(dirname "${TESTDATA}")" ]; then
		log_failure "exec %d failed; $(echo $(cat "${WORKDIR}/ret"))"
		return 1
	fi
	:
}

make_test_setup_pair 13 _exec_test_setup
disabled_test13() {
	local loc=$(make_mime_handler "rm %F")
	local x
	touch "${WORKDIR}/"{1,2,3,4,5}.${TEST_EXT}
	generic_exec_test "$loc" "${WORKDIR}"/{1,2,3,4,5}.${TEST_EXT}
	for x in "${WORKDIR}"/{1,2,3,4,5}.${TEST_EXT}; do
		[ ! -e "$x" ] && continue
		log_failure "exec %F parsing failed; $x wasn't wiped"
		return 1
	done
	:
}

make_test_setup_pair 14 _exec_test_setup
disabled_test14() {
	local loc=$(make_mime_handler "$LOGGER_SH ${WORKDIR}/ret %D")
	local x s d
	mkdir "${WORKDIR}"/{1,2}
	touch "${WORKDIR}"/{1,2}/foon.${TEST_EXT}
	generic_exec_test "$loc" "${WORKDIR}"/{1,2}/foon.${TEST_EXT}
	s="$(echo $(cat "${WORKDIR}/ret"))"
	s="${s//\/ /}"
	s="${s%%/}"
	d="${WORKDIR%%/}"
	d="${d}/1 ${d}/2"
	if [ "$s" != "$d" ]; then
		log_failure "%D parsing is broken; got $s; should have got $d"
		return 1
	fi
	:
}

make_test_setup_pair 15 _exec_test_setup
disabled_test15() {
	local loc=$(make_mime_handler "$LOGGER_SH ${WORKDIR}/ret %n")
	generic_exec_test "$loc" "${TESTDATA}"
	if [ "$(echo $(cat "${WORKDIR}/ret"))" != "$(basename "${TESTDATA}")" ]; then
		log_failure "exec %n parsing failed; expected $(basename "${TESTDATA}"), got $(<${WORKDIR}/ret)"
		return 1
	fi
	:
}

make_test_setup_pair 16 _exec_test_setup
disabled_test16() {
	local loc=$(make_mime_handler "$LOGGER_SH ${WORKDIR}/ret %N")
	touch "${WORKDIR}"/{1,2,3}.${TEST_EXT}
	generic_exec_test "$loc" "${WORKDIR}"/{1,2,3}.${TEST_EXT}
	if [ "$(echo $(sort "${WORKDIR}/ret"))" != "1 2 3" ]; then
		log_failure "exec %N parsing failed; expected 1 2 3, got $(<${WORKDIR}/ret)"
		return 1
	fi
	:
}

make_test_setup_pair 17 _exec_test_setup
test17() {
	local loc=$(make_mime_handler "$LOGGER_SH ${WORKDIR}/ret %u")
	generic_exec_test "$loc" "file://${TESTDATA}"
	local x="$(echo $(cat "${WORKDIR}/ret"))"
	if [ "$x" != "file://${TESTDATA}" ] && [ "$x" != "$TESTDATA" ]; then
		log_failure "exec %u parsing failed; expected file://${TESTDATA}, got $(<${WORKDIR}/ret)"
		return 1
	fi
	:
}

make_test_setup_pair 18 _exec_test_setup
disabled_test18() {
	local loc=$(make_mime_handler "$LOGGER_SH ${WORKDIR}/ret %N")
	touch "${WORKDIR}"/{1,2,3}.${TEST_EXT}
	local expected=$(echo "file://${WORKDIR}"{1,2,3}.${TEST_EXT})
	generic_exec_test "$loc" "file://${WORKDIR}/"{1,2,3}.${TEST_EXT}
	if [ "$(echo $(sort "${WORKDIR}/ret"))" != "$expected" ]; then
		log_failure "exec %N parsing failed; expected $expected, got $(<${WORKDIR}/ret)"
		return 1
	fi
	:
}

make_test_setup_pair 19 _exec_test_setup
test19() {
	local loc=$(make_mime_handler "$LOGGER_SH ${WORKDIR}/ret %% %f")
	generic_exec_test "$loc" "${TESTDATA}"
	if [ "$(echo -n $(cat "${WORKDIR}/ret"))" != "% ${TESTDATA}" ]; then
		log_failure "exec %% parsing failed; expected '% ${TESTDATA}', got '$(<${WORKDIR}/ret)'"
		return 1
	fi
	:
}

make_test_setup_pair 20 _exec_test_setup
test20() {
	local loc=$(make_mime_handler "$LOGGER_SH ${WORKDIR}/ret %% %f")
	generic_exec_test "$loc" "${TESTDATA}"
	if [ "$(echo -n $(cat "${WORKDIR}/ret"))" != "% ${TESTDATA}" ]; then
		log_failure "exec %% %f parsing failed; expected '% ${TESTDATA}', got '$(<${WORKDIR}/ret)'"
		return 1
	fi
	:
}

make_test_setup_pair 21 _exec_test_setup
test21() {
	local loc=$(make_mime_handler "bash -c 'pwd > ${WORKDIR}/baseline'")
	local test_dir=${WORKDIR}
	generic_exec_test "$loc" "${TESTDATA}"
	if [ ! -s "${WORKDIR}/baseline" ]; then
		log_failure "no results detected for Path key baseline test; failed"
		return 2
	fi
	# done on the extremely unlikely chance we execute the test in the directory it writes to
	if [ "${WORKDIR}" == "$(echo $(cat ${WORKDIR}/baseline))" ]; then
		test_dir="/tmp"
	fi
	loc=$(make_mime_handler "bash -c 'pwd > ${WORKDIR}/force'" "Path=${test_dir}")
	generic_exec_test "$loc" "${TESTDATA}"
	if [ "$(echo -n $(cat "${WORKDIR}/force"))" != "${test_dir}" ]; then
		log_failure "Path key parsing failed; expected '${test_dir}', got '$(<${WORKDIR}/force)'"
		return 1
	fi
	:
}

make_test_setup_pair 22 _exec_test_setup
test22() {
	local loc=$(make_mime_handler "$LOGGER_SH ${WORKDIR}/ret \"a b\" %f")
	generic_exec_test "$loc" "${TESTDATA}"
	local line_count=$(wc -l "${WORKDIR}/ret" | cut -d ' ' -f 1)
	if [ "$(echo -n $(cat "${WORKDIR}/ret"))" != "a b ${TESTDATA}" ]; then
		log_failure "exec %% %f parsing failed; expected '% ${TESTDATA}', got '$(<${WORKDIR}/ret)'"
		return 1
	elif [ "$line_count" != "2" ]; then
		log_failure "wrong line count; got $line_count args, expected two"
		return 1
	fi
	:
}

make_test_setup_pair 23 _exec_test_setup
test23() {
	local loc=$(make_mime_handler "$LOGGER_SH ${WORKDIR}/ret \\\\\\\\'a b %f")
	generic_exec_test "$loc" "${TESTDATA}"
	local line_count=$(wc -l "${WORKDIR}/ret" | cut -d ' ' -f 1)
	if [ "$(echo -n $(cat "${WORKDIR}/ret"))" != "'a b ${TESTDATA}" ]; then
		log_failure "exec '\'a b %f' parsing failed; expected '\'a b ${TESTDATA}', got '$(<${WORKDIR}/ret)'"
		return 1
	elif [ "$line_count" != "3" ]; then
		log_failure "wrong line count; got $line_count args, expected 3"
		return 1
	fi
	:
}

# disabled since "%f" is still up in the air; likely to die however
make_test_setup_pair 24 _exec_test_setup
disabled_test24() {
	local loc=$(make_mime_handler "$LOGGER_SH ${WORKDIR}/ret \"%f\"")
	generic_exec_test "$loc" "${TESTDATA}"
	if [ "$(echo -n $(cat "${WORKDIR}/ret"))" != "${TESTDATA}" ]; then
		log_failure "exec '\"%f\"' parsing failed; expected '${TESTDATA}', got '$(<${WORKDIR}/ret)'"
		return 1
	fi
	:
}

make_test_setup_pair 25 _exec_test_setup
disabled_test25() {
	local loc=$(make_mime_handler "$LOGGER_SH ${WORKDIR}/ret \"a b %f\"")
	generic_exec_test "$loc" "${TESTDATA}"
	local line_count=$(wc -l "${WORKDIR}/ret" | cut -d ' ' -f 1)
	if [ "$(echo -n $(cat "${WORKDIR}/ret"))" != "a b ${TESTDATA}" ]; then
		log_failure "exec 'a b %f' parsing failed; expected 'a b ${TESTDATA}', got '$(<${WORKDIR}/ret)'"
		return 1
	elif [ "$line_count" != "1" ]; then
		log_failure "wrong line count; got $line_count args, expected 1"
		return 1
	fi
	:
}

make_test_setup_pair 26 _exec_test_setup
test26() {
	local loc=$(make_mime_handler "$LOGGER_SH ${WORKDIR}/ret "'"\\\\\\\\" %f')
	generic_exec_test "$loc" "${TESTDATA}"
	if [ "$(echo -n $(cat "${WORKDIR}/ret"))" != "\\ ${TESTDATA}" ]; then
		log_failure "exec '"\\" %f' parsing failed; expected '"\\" ${TESTDATA}', got '$(<${WORKDIR}/ret)'"
		return 1
	fi
	:
}

make_test_setup_pair 27 _exec_test_setup
test27() {
	local loc=$(make_mime_handler "$LOGGER_SH ${WORKDIR}/ret %f a")
	generic_exec_test "$loc" "${TESTDATA}"
	if [ "$(echo -n $(cat "${WORKDIR}/ret"))" != "${TESTDATA} a" ]; then
		log_failure "exec '%f a' parsing failed; expected '${TESTDATA} a', got '$(<${WORKDIR}/ret)'"
		return 1
	fi
	:
}



DATADIR=$(dirname "$(readlink -f "$0")")
TESTDIR="/tmp/desktop-test-$(date +%s)"
mkdir -p "$TESTDIR"
readonly TESTMIME="text/x-desktop-spec-test"
readonly TEST_EXT="spec_desktop_test"
readonly TESTDATA="${DATADIR}/data.${TEST_EXT}"
readonly SLEEPDELAY=${SLEEPDELAY:-3}
readonly LOGGER_SH="${DATADIR}/logger.sh"

if ! which xdg-open &> /dev/null; then
	echo "xdg-utils is required for these tests";
	exit 1
fi

if [ -z "$*" ]; then
	tests=$(declare -F | cut -d ' ' -f 3 | sort | egrep '^test[0-9]+$' | sed -e 's:^test::')
else
	tests="$*"
fi
for count in ${tests}; do
	eval tp${count}$'() {
		local ret
		export WORKDIR="${TESTDIR}/test'${count}$'"
		rm -rf "${WORKDIR}"
		mkdir -p "${WORKDIR}"
		export TESTCASE=test'${count}$'
		test'${count}$'_setup
		( test'${count}$'; )
		ret=$?
		if [ 0 == "$ret" ]; then
			log_pass
			rm -rf "${WORKDIR}"
		fi
		test'${count}$'_teardown
		unset TESTCASE
		return $(( $ret ))
	}'
done

if [ -z "$TET_RUN" ]; then
	ret=0
	[ -n "${DEBUG}" ] && set -x
	for count in ${tests}; do
		tp${count}
		ret=$(( ${ret} + ${?} ))
	done
	if [ "$ret2" != "0" ]; then
		echo "test data is accessible in ${TESTDIR}"
	fi
	exit $(( $ret ))
fi
